import { Component, ElementRef, EventEmitter, Injectable, Input, NgModule, NgZone, Optional, Output, ViewChild, animate, forwardRef, style, transition, trigger } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, NG_VALUE_ACCESSOR, ReactiveFormsModule } from '@angular/forms';
import moment from 'moment';

class AngularDatepickerOptions {
    constructor() {
        this.format = 'DD.MM.YYYY';
        this.viewFormat = 'DD.MM.YYYY';
        this.firstWeekdaySunday = false;
        this.todayButtonEnabled = true;
        this.todayHighlight = true;
        this.weekendHighlight = true;
        this.closeOnSelect = true;
    }
}
AngularDatepickerOptions.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
AngularDatepickerOptions.ctorParameters = () => [];

class DatePickerComponent {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    constructor(elRef, opts) {
        this.elRef = elRef;
        this.opts = opts;
        this.options = {};
        this.date = moment();
        this.viewDate = null;
        this.days = [];
        this.onTouchedCallback = () => { };
        this.onChangeCallback = () => { };
        this.closePickerOnOutsideClick = (event) => this.close(event);
        this.closePickerOnTab = (event) => this.closeOnTab(event);
    }
    /**
     * @return {?}
     */
    get value() {
        return this.viewDate;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        let /** @type {?} */ date = (value instanceof moment) ? value : moment(value, this.options.format);
        this.viewDate = date.isValid() ? date.format(this.options.viewFormat) : value;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    onDateSelected(date) {
        this.value = date;
        this.onChangeCallback(date.format(this.options.format));
        if (this.options.closeOnSelect) {
            this.close();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.opts) {
            if (typeof this.options.format === 'undefined') {
                this.options.format = this.opts.format;
            }
            if (typeof this.options.viewFormat === 'undefined') {
                this.options.viewFormat = this.opts.viewFormat;
            }
            if (typeof this.options.modalMediaQuery === 'undefined') {
                this.options.modalMediaQuery = this.opts.modalMediaQuery;
            }
            if (typeof this.options.firstWeekdaySunday === 'undefined') {
                this.options.firstWeekdaySunday = this.opts.firstWeekdaySunday;
            }
            else {
                this.options.firstWeekdaySunday = this.options.firstWeekdaySunday.toString() === 'true';
            }
            if (typeof this.options.todayString === 'undefined') {
                this.options.todayString = this.opts.todayString;
            }
            if (typeof this.options.todayButtonEnabled === 'undefined') {
                this.options.todayButtonEnabled = this.opts.todayButtonEnabled;
            }
            else {
                this.options.todayButtonEnabled = this.options.todayButtonEnabled.toString() === 'true';
            }
            if (typeof this.options.todayHighlight === 'undefined') {
                this.options.todayHighlight = this.opts.todayHighlight;
            }
            else {
                this.options.todayHighlight = this.options.todayHighlight.toString() === 'true';
            }
            if (typeof this.options.weekendHighlight === 'undefined') {
                this.options.weekendHighlight = this.opts.weekendHighlight;
            }
            else {
                this.options.weekendHighlight = this.options.weekendHighlight.toString() === 'true';
            }
            if (typeof this.options.closeOnSelect === 'undefined') {
                this.options.closeOnSelect = this.opts.closeOnSelect;
            }
            else {
                this.options.closeOnSelect = this.options.closeOnSelect.toString() === 'true';
            }
        }
        this.datePicker.mergeOptions(this.options);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.opened) {
            this.datePicker.open();
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.viewDate = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    /**
     * @return {?}
     */
    toggle() {
        if (!this.datePicker.opened) {
            this.open();
        }
        else {
            // this.close();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    closeOnTab(event) {
        if (event.keyCode === 9) {
            if (this.viewDate) {
                this.value = moment(this.viewDate, this.options.viewFormat);
            }
            this.onTouchedCallback();
            this.close();
        }
    }
    /**
     * @param {?=} event
     * @return {?}
     */
    open(event) {
        if (event) {
            event.preventDefault();
        }
        let /** @type {?} */ date = moment(this.value, this.options.viewFormat);
        this.datePicker.inputDate = date.isValid() ? date : null;
        this.datePicker.open();
        window.addEventListener('click', this.closePickerOnOutsideClick, true);
        window.addEventListener('keydown', this.closePickerOnTab, true);
    }
    /**
     * @param {?=} event
     * @return {?}
     */
    close(event) {
        if (event && this.elRef.nativeElement.contains(event.target)) {
            return;
        }
        this.onTouchedCallback();
        window.removeEventListener('keydown', this.closePickerOnTab, true);
        window.removeEventListener('click', this.closePickerOnOutsideClick, true);
        this.datePicker.close();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        window.removeEventListener('keydown', this.closePickerOnTab, true);
        window.removeEventListener('click', this.closePickerOnOutsideClick, true);
    }
}
DatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: '',
                template: ''
            },] },
];
/**
 * @nocollapse
 */
DatePickerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, },
];
DatePickerComponent.propDecorators = {
    'class': [{ type: Input },],
    'opened': [{ type: Input },],
    'options': [{ type: Input },],
};

class DatePickerContainerComponent {
    /**
     * @param {?} elRef
     * @param {?} zone
     */
    constructor(elRef, zone) {
        this.elRef = elRef;
        this.zone = zone;
        this.onDateSelected = new EventEmitter();
        this.opened = false;
        this.date = moment();
        this.days = [];
        this.options = {};
        this.watchMediaQuery = (mql) => {
            this.zone.run(() => {
                this.mqMatches = this.mq.matches ? true : false;
            });
        };
    }
    /**
     * @param {?} options
     * @return {?}
     */
    mergeOptions(options) {
        Object.assign(this.options, options);
    }
    /**
     * @param {?=} selectedMonthFlag
     * @return {?}
     */
    generateCalendar(selectedMonthFlag) {
        this.weekDayNames = moment.weekdaysShort();
        if (!this.firstWeekdaySunday) {
            this.weekDayNames.splice(6, 0, this.weekDayNames.splice(0, 1)[0]);
        }
        this.date = selectedMonthFlag && this.inputDate ? this.inputDate.clone() : moment(this.date);
        let /** @type {?} */ month = this.date.month();
        let /** @type {?} */ firstDay = this.date.clone().startOf('month');
        let /** @type {?} */ offset;
        if (this.firstWeekdaySunday) {
            offset = firstDay.day() === 0 ? 7 : firstDay.day();
        }
        else {
            offset = firstDay.day() === 0 ? 6 : firstDay.day() - 1;
        }
        let /** @type {?} */ daysInMonth = this.date.daysInMonth();
        let /** @type {?} */ rows = Math.ceil((offset + daysInMonth) / 7);
        this.weeks = [];
        let /** @type {?} */ currentDate = firstDay.subtract(offset, 'day');
        let /** @type {?} */ selectedDate = this.inputDate;
        for (let /** @type {?} */ i = 1; i <= rows; i += 1) {
            let /** @type {?} */ days = [];
            for (let /** @type {?} */ i = 1; i <= 7; i += 1) {
                let /** @type {?} */ today = (moment().isSame(currentDate, 'day') && moment().isSame(currentDate, 'month')) ? true : false;
                let /** @type {?} */ weekend = (moment(currentDate).day() == 6) || (moment(currentDate).day() == 0);
                let /** @type {?} */ selected = (selectedDate && selectedDate.isSame(currentDate, 'day')) ? true : false;
                days.push({
                    day: currentDate.date(),
                    month: currentDate.month() + 1,
                    year: currentDate.year(),
                    enabled: true,
                    differentMonth: month === currentDate.month(),
                    today: today,
                    weekend: weekend,
                    selected: selected
                });
                currentDate.add(1, 'day');
            }
            this.weeks.push({ rowId: i, days: days });
        }
    }
    /**
     * @return {?}
     */
    setToday() {
        let /** @type {?} */ today = moment();
        this.inputDate = today;
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    }
    /**
     * @param {?} e
     * @param {?} d
     * @return {?}
     */
    selectDate(e, d) {
        e.preventDefault();
        this.inputDate = moment(`${d.day}.${d.month}.${d.year}`, 'DD.MM.YYYY');
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    }
    /**
     * @return {?}
     */
    prevMonth() {
        this.date = this.date.subtract(1, 'month');
        this.generateCalendar();
    }
    /**
     * @return {?}
     */
    nextMonth() {
        this.date = this.date.add(1, 'month');
        this.generateCalendar();
    }
    /**
     * @return {?}
     */
    open() {
        this.opened = true;
        this.generateCalendar(true);
        if (this.options.modalMediaQuery) {
            this.mq = window.matchMedia(`(${this.options.modalMediaQuery})`);
            this.mqMatches = this.mq.matches ? true : false;
            this.mq.addListener(this.watchMediaQuery);
        }
    }
    /**
     * @return {?}
     */
    close() {
        this.opened = false;
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    }
}
DatePickerContainerComponent.decorators = [
    { type: Component, args: [{
                selector: 'jb-datepicker-container',
                template: `
      <div class="calendar-wrapper" [class.modal]="mqMatches" *ngIf="opened" [@contentAnimation]="opened">
        <div class="overlay" (click)="close()"></div>
        <div class="calendar-container">
          <div class="calendar-top">
            <div class="top-chevron" (click)="prevMonth()">
              <svg fill="#FFFFFF" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg">
                <path d="M15.41 7.41L14 6l-6 6 6 6 1.41-1.41L10.83 12z" />
                <path d="M0 0h24v24H0z" fill="none" />
              </svg>
            </div>
            <div class="top-month-year">
              <span>{{ date.format('MMMM YYYY') }}</span>
            </div>
            <div class="top-chevron" (click)="nextMonth()">
              <svg fill="#FFFFFF" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg">
                <path d="M10 6L8.59 7.41 13.17 12l-4.58 4.59L10 18l6-6z" />
                <path d="M0 0h24v24H0z" fill="none" />
              </svg>
            </div>
          </div>
          <table>
            <thead>
              <tr class="day-names">
                <th *ngFor="let name of weekDayNames">{{name}}</th>
              </tr>
            </thead>
            <tbody class="days" [class.weekend-highlight]="options.weekendHighlight" [class.today-highlight]="options.todayHighlight">
              <tr *ngFor="let week of weeks">
                <td *ngFor="let d of week.days" class="day" [class.weekend]="d.weekend">
                  <span (click)="selectDate($event, d)" [class.today]="d.today" [class.different-month]="!d.differentMonth" [class.is-active]="d.selected">{{d.day}}</span>
                </td>
              </tr>
              <tr>
                <td colspan="7" class="today-button-wrapper">
                  <button type="button" (click)="setToday()">{{options.todayString}}</button>
                </td>
              </tr>
            </tbody>
          </table>
        </div>
      </div>
    `,
                styles: [`
      :host .calendar-wrapper{position:absolute;z-index:9999;min-width:200px}:host .calendar-wrapper .overlay{display:none}:host .calendar-wrapper .calendar-container{min-width:90%;z-index:999;background-color:#fff;box-shadow:0 1px 3px rgba(0,0,0,0.12),0 1px 2px rgba(0,0,0,0.24)}:host .calendar-wrapper .calendar-container .calendar-top{width:100%;min-height:40px;color:#fff;background-color:#613bbb;line-height:40px;display:flex;align-items:center;justify-content:space-between}:host .calendar-wrapper .calendar-container .calendar-top .top-chevron{cursor:pointer;padding:0.5em;outline:none}:host .calendar-wrapper .calendar-container .calendar-top .top-chevron svg{vertical-align:middle}:host .calendar-wrapper .calendar-container .calendar-top .top-month-year{padding:0.5em;outline:none}:host .calendar-wrapper .calendar-container .calendar-top .top-month-year span{text-align:center;font-size:1.2em;vertical-align:middle}:host .calendar-wrapper .calendar-container table{width:100%;border-collapse:collapse;text-align:center}:host .calendar-wrapper .calendar-container .day-names{background:#7C4DFF;padding-left:5px;padding-right:5px}:host .calendar-wrapper .calendar-container .day-names th{width:calc(100% / 7);font-size:1em;text-align:center;padding-top:5px;padding-bottom:5px;color:#fff}:host .calendar-wrapper .calendar-container .days{margin-left:5px;margin-right:5px}:host .calendar-wrapper .calendar-container .days.weekend-highlight .day.weekend{background-color:rgba(0,0,0,0.1)}:host .calendar-wrapper .calendar-container .days.today-highlight .day span.today{font-weight:bold;color:#fff;background:rgba(97,59,187,0.4)}:host .calendar-wrapper .calendar-container .days .day{cursor:pointer;text-align:center}:host .calendar-wrapper .calendar-container .days .day span{font-size:1em;display:inline-block;text-align:center;height:2em;width:2em;line-height:2em;border-radius:50%;margin:2px}:host .calendar-wrapper .calendar-container .days .day span.different-month{color:#ccc}:host .calendar-wrapper .calendar-container .days .day span:hover,:host .calendar-wrapper .calendar-container .days .day span.is-active,:host .calendar-wrapper .calendar-container .days .day span.is-active.today{background:#613bbb;color:#fff}:host .calendar-wrapper .calendar-container .today-button-wrapper{padding-bottom:5px;padding-top:5px}:host .calendar-wrapper .calendar-container .today-button-wrapper button{display:flex;margin:auto;align-items:center;align-self:center;justify-content:center;margin-bottom:0;font-weight:400;text-align:center;vertical-align:middle;cursor:pointer;background-image:none;border:1px solid transparent;white-space:nowrap;padding:5px 10px;font-size:1em;line-height:1.4;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;outline:none}:host .calendar-wrapper.modal{display:flex;flex-flow:column;position:fixed;top:0;left:0;width:100%;height:100%;justify-content:center;align-items:center}:host .calendar-wrapper.modal .overlay{position:absolute;top:0;left:0;width:100%;height:100%;display:flex;z-index:900;justify-content:center;align-items:center;background-color:rgba(0,0,0,0.7)}
    `],
                animations: [
                    trigger('contentAnimation', [
                        transition('void => *', [
                            style({ opacity: '0' }),
                            animate('200ms ease-in')
                        ]),
                        transition('* => void', [
                            animate('300ms ease-out', style({ opacity: '0' }))
                        ])
                    ])
                ]
            },] },
];
/**
 * @nocollapse
 */
DatePickerContainerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
];
DatePickerContainerComponent.propDecorators = {
    'onDateSelected': [{ type: Output },],
};

const CALENDAR_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => InputDatePickerComponent),
    multi: true
};
class InputDatePickerComponent extends DatePickerComponent {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    constructor(elRef, opts) {
        super(elRef, opts);
        this.elRef = elRef;
        this.opts = opts;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    viewDateChanged(event) {
        if (event && event.length > 0) {
            this.onChangeCallback(event);
        }
        else {
            this.onChangeCallback(null);
        }
    }
    /**
     * @param {?=} event
     * @return {?}
     */
    toggle(event) {
        if (event) {
            event.preventDefault();
            event.stopPropagation();
        }
        if (!this.datePicker.opened) {
            if (this.viewDate) {
                this.inputEl.nativeElement.setSelectionRange(0, this.viewDate.length);
            }
            this.open(event);
        }
        else {
            // this.close();
        }
    }
}
InputDatePickerComponent.decorators = [
    { type: Component, args: [{
                // moduleId: module.id,
                selector: 'jb-input-datepicker',
                template: `
      <div class="datepicker-wrapper">
        <input type="text" #input [(ngModel)]="viewDate" (click)="toggle($event)" (ngModelChange)="viewDateChanged($event)" [placeholder]="options.viewFormat">
        <jb-datepicker-container (onDateSelected)="onDateSelected($event)"></jb-datepicker-container>
      </div>
    `,
                styles: [`
      :host{position:relative;width:100%}:host .datepicker-wrapper{position:relative;display:flex;flex-flow:column}:host .datepicker-wrapper input{width:100%}
    `],
                providers: [CALENDAR_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
InputDatePickerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, decorators: [{ type: Optional },] },
];
InputDatePickerComponent.propDecorators = {
    'datePicker': [{ type: ViewChild, args: [DatePickerContainerComponent,] },],
    'inputEl': [{ type: ViewChild, args: ['input',] },],
    'options': [{ type: Input },],
};

const CALENDAR_VALUE_ACCESSOR$1 = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => TextDatePickerComponent),
    multi: true
};
class TextDatePickerComponent extends DatePickerComponent {
    /**
     * @param {?} elRef
     * @param {?} opts
     */
    constructor(elRef, opts) {
        super(elRef, opts);
        this.elRef = elRef;
        this.opts = opts;
    }
}
TextDatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'jb-text-datepicker',
                template: `
    <div class="datepicker-wrapper">
      <p (click)="toggle()">{{viewDate || 'DATUM'}}</p>
      <jb-datepicker-container (onDateSelected)="onDateSelected($event)"></jb-datepicker-container>
    </div>
  `,
                styles: [`
    :host{position:relative;width:100%}:host .datepicker-wrapper{position:relative;display:flex;flex-flow:column}:host .datepicker-wrapper input{width:100%}
  `],
                providers: [CALENDAR_VALUE_ACCESSOR$1]
            },] },
];
/**
 * @nocollapse
 */
TextDatePickerComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: AngularDatepickerOptions, decorators: [{ type: Optional },] },
];
TextDatePickerComponent.propDecorators = {
    'datePicker': [{ type: ViewChild, args: [DatePickerContainerComponent,] },],
    'options': [{ type: Input },],
};

class AngularDatepickerModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: AngularDatepickerModule,
            providers: [
                { provide: AngularDatepickerOptions, useValue: config }
            ]
        };
    }
}
AngularDatepickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    ReactiveFormsModule,
                    FormsModule
                ],
                declarations: [
                    DatePickerComponent,
                    DatePickerContainerComponent,
                    InputDatePickerComponent,
                    TextDatePickerComponent
                ],
                providers: [],
                exports: [
                    DatePickerComponent,
                    DatePickerContainerComponent,
                    InputDatePickerComponent,
                    TextDatePickerComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
AngularDatepickerModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { DatePickerComponent, DatePickerContainerComponent, InputDatePickerComponent, TextDatePickerComponent, AngularDatepickerOptions, AngularDatepickerModule, CALENDAR_VALUE_ACCESSOR as ɵa, CALENDAR_VALUE_ACCESSOR$1 as ɵb };
